@@ -21,6 +21,10 @@
 #include "base_db.h"
 #include "application.h" //App_Glom.
 #include "appstate.h"
+#include "standard_table_prefs_fields.h"
+#include "document/document_glom.h"
+#include "data_structure/glomconversions.h"
+#include "mode_data/dialog_choose_field.h"
 #include <glibmm/i18n.h>
 //#include <libgnomeui/gnome-app-helper.h>
 
@@ -35,19 +39,20 @@ Base_DB::~Base_DB()
 {
 }
 
-void Base_DB::init_db_details()
+bool Base_DB::init_db_details()
 {
-  fill_from_database();
+  return fill_from_database();
 }
 
-void Base_DB::refresh_data_from_database()
+bool Base_DB::refresh_data_from_database()
 {
-  fill_from_database();
+  return fill_from_database();
 }
 
-void Base_DB::fill_from_database()
+bool Base_DB::fill_from_database()
 {
   //m_AddDel.remove_all();
+  return true;
 }
 
 void Base_DB::fill_end()
@@ -133,6 +138,10 @@ Glib::RefPtr<Gnome::Gda::DataModel> Base_DB::Query_execute(const Glib::ustring&
       handle_error();
     }
   }
+  else
+  {
+    g_warning("Base_DB::Query_execute(): No connection yet.");
+  }
 
   return result;
 }
@@ -148,8 +157,18 @@ void Base_DB::load_from_document()
   }
 }
 
+bool Base_DB::get_table_exists_in_database(const Glib::ustring& table_name) const
+{
+  //TODO_Performance
 
-Base_DB::type_vecStrings Base_DB::get_table_names()
+  type_vecStrings tables = get_table_names();
+  type_vecStrings::const_iterator iterFind = std::find(tables.begin(), tables.end(), table_name);
+  bool result = (iterFind != tables.end());
+
+  return result;
+}
+
+Base_DB::type_vecStrings Base_DB::get_table_names() const
 {
   type_vecStrings result;
 
@@ -635,6 +654,16 @@ Privileges Base_DB::get_table_privileges(const Glib::ustring& group_name, const
 {
   Privileges result;
 
+  if(group_name == GLOM_STANDARD_GROUP_NAME_DEVELOPER)
+  {
+    //Always give developers full access:
+    result.m_view = true;
+    result.m_edit = true;
+    result.m_create = true;
+    result.m_delete = true;
+    return result;
+  }
+
   //Get the permissions:
   Glib::ustring strQuery = "SELECT pg_class.relacl FROM pg_class WHERE pg_class.relname = '" + table_name + "'";
   Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute(strQuery);
@@ -700,9 +729,108 @@ Privileges Base_DB::get_table_privileges(const Glib::ustring& group_name, const
     }
   }
 
+  g_warning("get_table_privileges(group_name=%s, table_name=%s) returning: %d", group_name.c_str(), table_name.c_str(), result.m_create);
   return result;
 }
 
+void Base_DB::add_standard_tables() const
+{
+  //Name, address, etc:
+  if(!get_table_exists_in_database(GLOM_STANDARD_TABLE_PREFS_TABLE_NAME))
+  {
+    TableInfo prefs_table_info;
+    prefs_table_info.m_name = GLOM_STANDARD_TABLE_PREFS_TABLE_NAME;
+    prefs_table_info.m_hidden = true;
+
+    Document_Glom::type_vecFields pref_fields;
+
+    Field primary_key; //It's not used, because there's only one record, but we must have one.
+    primary_key.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ID);
+    primary_key.set_glom_type(Field::TYPE_NUMERIC);
+    pref_fields.push_back(primary_key);
+
+    Field field_name;
+    field_name.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_NAME);
+    field_name.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_name);
+
+    Field field_org_name;
+    field_org_name.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_NAME);
+    field_org_name.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_name);
+
+    Field field_org_address_street;
+    field_org_address_street.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET);
+    field_org_address_street.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_street);
+
+    Field field_org_address_street2;
+    field_org_address_street2.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET2);
+    field_org_address_street2.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_street2);
+
+    Field field_org_address_town;
+    field_org_address_town.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_TOWN);
+    field_org_address_town.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_town);
+
+    Field field_org_address_county;
+    field_org_address_county.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTY);
+    field_org_address_county.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_county);
+
+    Field field_org_address_country;
+    field_org_address_country.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTRY);
+    field_org_address_country.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_country);
+
+    Field field_org_address_postcode;
+    field_org_address_postcode.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE);
+    field_org_address_postcode.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_postcode);
+
+    bool test = create_table(prefs_table_info, pref_fields);
+
+    if(test)
+    {
+      //Add the single record:
+      Query_execute("INSERT INTO " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "(" GLOM_STANDARD_TABLE_PREFS_FIELD_ID ") VALUES (1)");
+    }
+  }
+
+  //Auto-increment next values:
+  if(!get_table_exists_in_database(GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME))
+  {
+    TableInfo table_info;
+    table_info.m_name = GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME;
+    table_info.m_hidden = true;
+
+    Document_Glom::type_vecFields fields;
+
+    Field primary_key; //It's not used, because there's only one record, but we must have one.
+    primary_key.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_ID);
+    primary_key.set_glom_type(Field::TYPE_NUMERIC);
+    fields.push_back(primary_key);
+
+    Field field_table_name;
+    field_table_name.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME);
+    field_table_name.set_glom_type(Field::TYPE_TEXT);
+    fields.push_back(field_table_name);
+
+    Field field_field_name;
+    field_field_name.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME);
+    field_field_name.set_glom_type(Field::TYPE_TEXT);
+    fields.push_back(field_field_name);
+
+    Field field_next_value;
+    field_next_value.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE);
+    field_next_value.set_glom_type(Field::TYPE_TEXT);
+    fields.push_back(field_next_value);
+
+    create_table(table_info, fields);
+  }
+}
+
 void Base_DB::add_standard_groups()
 {
   //Add the glom_developer group if it does not exist:
@@ -713,13 +841,17 @@ void Base_DB::add_standard_groups()
   if(iterFind == vecGroups.end())
   {
     Query_execute("CREATE GROUP " GLOM_STANDARD_GROUP_NAME_DEVELOPER);
-    Privileges priv_ignored;
+    Privileges priv_devs;
+    priv_devs.m_view = true;
+    priv_devs.m_edit = true;
+    priv_devs.m_create = true;
+    priv_devs.m_delete = true;
 
     Document_Glom::type_listTableInfo table_list = get_document()->get_tables();
 
     for(Document_Glom::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
     {
-      set_table_privileges(devgroup, iter->m_name, priv_ignored, true /* developer privileges */);
+      set_table_privileges(devgroup, iter->m_name, priv_devs, true /* developer privileges */);
     }
 
     //Make sure that it is in the database too:
@@ -730,6 +862,68 @@ void Base_DB::add_standard_groups()
   }
 }
 
+Gnome::Gda::Value Base_DB::auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name)
+{
+  Gnome::Gda::Value value;
+
+  const Glib::ustring sql_query = "SELECT " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME ".next_value FROM " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME
+   " WHERE " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME " = '" + table_name + "' AND "
+             GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME " = '" + field_name + "'";
+
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+  if(!datamodel || (datamodel->get_n_rows() == 0))
+  {
+    //Start with zero:
+
+    //Insert the row if it's not there.
+    const Glib::ustring sql_query = "INSERT INTO " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME " ("
+      GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME ", " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME ", " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE
+      ") VALUES ('" + table_name + "', '" + field_name + "', 0)";
+
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    if(!datamodel)
+    {
+      g_warning("Base_DB::auto_increment_insert_first_if_necessary(): INSERT of new row failed.");
+    }
+
+    //GdaNumeric is a pain, so we take a short-cut:
+    bool success = false;
+    value = GlomConversions::parse_value(Field::TYPE_NUMERIC, "0", success, true /* iso_format */);
+  }
+  else
+  {
+    //Return the value so that a calling function does not need to do a second SELECT.
+    value = datamodel->get_value_at(0, 0);
+  }
+
+  return value;
+}
+
+Gnome::Gda::Value Base_DB::get_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name)
+{
+  const Gnome::Gda::Value result = auto_increment_insert_first_if_necessary(table_name, field_name);
+  long num_result = 0;
+  num_result = util_decimal_from_string(result.to_string());
+
+
+  //Increment the next_value:
+  ++num_result;
+  const Gnome::Gda::Value next_value = GlomConversions::parse_value(num_result);
+
+  const Glib::ustring sql_query = "UPDATE " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME " SET "
+      GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE " = " + next_value.to_string() +
+      " WHERE " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME " = '" + table_name + "' AND "
+                GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME " = '" + field_name + "'";
+
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+  if(!datamodel)
+  {
+    g_warning("Base_DB::get_next_auto_increment_value(): Increment failed.");
+  }
+
+  return result;
+}
+
 
 Glib::ustring Base_DB::string_trim(const Glib::ustring& str, const Glib::ustring& to_remove)
 {
@@ -877,3 +1071,180 @@ Privileges Base_DB::get_current_privs(const Glib::ustring& table_name) const
 
   return result;
 }
+
+SystemPrefs Base_DB::get_database_preferences() const
+{
+  //if(get_userlevel() == AppState::USERLEVEL_DEVELOPER)
+  //  add_standard_tables();
+
+  SystemPrefs result;
+
+  const Glib::ustring sql_query = "SELECT "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_NAME ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_NAME ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET2 ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_TOWN ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTY ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTRY ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE
+      " FROM " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME;
+
+  try
+  {
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    if(datamodel && (datamodel->get_n_rows() != 0))
+    {
+      result.m_name = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(0, 0));
+      result.m_org_name = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(1, 0));
+      result.m_org_address_street = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(2, 0));
+      result.m_org_address_street2 = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(3, 0));
+      result.m_org_address_town = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(4, 0));
+      result.m_org_address_county = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(5, 0));
+      result.m_org_address_country = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(6, 0));
+      result.m_org_address_postcode = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(7, 0));      }
+  }
+  catch(const std::exception& ex)
+  {
+    std::cerr << "Base_DB::get_database_preferences(): exception: " << ex.what() << std::endl;
+  }
+
+  return result;
+}
+
+void Base_DB::set_database_preferences(const SystemPrefs& prefs)
+{
+  if(get_userlevel() == AppState::USERLEVEL_DEVELOPER)
+    add_standard_tables();
+
+  const Glib::ustring sql_query = "UPDATE " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME " SET "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_NAME " = '" + prefs.m_name + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_NAME " = '" + prefs.m_org_name + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET " = '" + prefs.m_org_address_street + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET2 " = '" + prefs.m_org_address_street2 + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_TOWN " = '" + prefs.m_org_address_town + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTY " = '" + prefs.m_org_address_county + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTRY " = '" + prefs.m_org_address_country + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE " = '" + prefs.m_org_address_postcode + "'"
+      " WHERE " GLOM_STANDARD_TABLE_PREFS_FIELD_ID " = 1";
+
+    try
+    {
+      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    }
+    catch(const std::exception& ex)
+    {
+      std::cerr << "Base_DB::set_database_preferences(): exception: " << ex.what() << std::endl;
+    }
+}
+
+bool Base_DB::create_table(const TableInfo& table_info, const Document_Glom::type_vecFields& fields) const
+{
+  bool table_creation_succeeded = false;
+
+  //Create SQL to describe all fields in this table:
+  Glib::ustring sql_fields;
+  for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+  {
+    //Create SQL to describe this field:
+    Field field = *iter;
+
+    //The field has no gda type, so we set that:
+    //This usually comes from the database, but that's a bit strange.
+    Gnome::Gda::FieldAttributes info = field.get_field_info();
+    info.set_gdatype( Field::get_gda_type_for_glom_type(field.get_glom_type()) );
+    field.set_field_info(info);
+
+    Glib::ustring sql_field_description = field.get_name() + " " + field.get_sql_type();
+
+    if(field.get_primary_key())
+      sql_field_description += " NOT NULL  PRIMARY KEY";
+
+    //Append it:
+    if(!sql_fields.empty())
+      sql_fields += ", ";
+
+    sql_fields += sql_field_description;
+  }
+
+  if(sql_fields.empty())
+  {
+    g_warning("Base_Db::create_table::create_table(): sql_fields is empty.");
+  }
+
+  //Actually create the table
+  try
+  {
+    Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute( "CREATE TABLE \"" + table_info.get_name() + "\" (" + sql_fields + ")" );
+    if(!data_model)
+      table_creation_succeeded = false;
+    else
+      table_creation_succeeded = true;
+  }
+  catch(const ExceptionConnection& ex)
+  {
+    table_creation_succeeded = false;
+  }
+
+  return table_creation_succeeded;
+}
+
+bool Base_DB::offer_field_list(LayoutItem_Field& field, const Glib::ustring& table_name)
+{
+  bool result = false;
+
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field");
+
+    Dialog_ChooseField* dialog = 0;
+    refXml->get_widget_derived("dialog_choose_field", dialog);
+
+    if(dialog)
+    {
+      dialog->set_document(get_document(), table_name, field);
+      //TODO: dialog->set_transient_for(*get_app_window());
+      int response = dialog->run();
+      if(response == Gtk::RESPONSE_OK)
+      {
+        //Get the chosen field:
+        result = dialog->get_field_chosen(field);
+      }
+
+      delete dialog;
+    }
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  return result;
+}
+
+void Base_DB::fill_full_field_details(const Glib::ustring& parent_table_name, LayoutItem_Field& layout_item)
+{
+  Glib::ustring table_name = parent_table_name;
+
+  if(layout_item.get_has_relationship_name())
+    table_name = layout_item.m_relationship.get_to_table();
+
+  get_document()->get_field(table_name, layout_item.get_name(), layout_item.m_field);
+}
+
+Glib::ustring Base_DB::get_layout_item_table_name(const LayoutItem_Field& layout_item, const Glib::ustring& table_name)
+{
+  if(!layout_item.get_has_relationship_name())
+    return table_name;
+  else
+  {
+    const Glib::ustring relationship_name = layout_item.get_relationship_name();
+    Relationship relationship; //TODO: We should not need to do this. It should be updated in the LayoutItem_Field already.
+    Document_Glom* document = get_document();
+    bool test = document->get_relationship(table_name, relationship_name, relationship);
+    if(test)
+     return relationship.get_to_table();
+  }
+
+  return Glib::ustring();
+}
