@@ -141,8 +141,14 @@ public:
    */
   Glib::ustring sql(const Gnome::Gda::Value& value) const;
 
+  /** Get a suitable operator to use when finding records.
+   * For instance, == for numbers, or LIKE for text.
+   */
+  Glib::ustring sql_find_operator() const;
+
   Glib::ustring get_calculation() const;
   void set_calculation(const Glib::ustring& calculation);
+  bool get_has_calculation() const;
 
   /* Discover what fields are used in the calculation,
    * so the value can be recalculated when their values change.
